/* ccgo: sgf/prop.cc
 *
 * Copyright 2002 Chun-Chung Chen <cjj@u.washington.edu>
 *
 * This program is free software; you can redistribute it and'or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include "prop.hh"
#include "sgf.hh"
#include <cctype>
#include <sstream>
using namespace std;
using namespace sgf;

bool Prop::load(istream & f)
{
	id = "";
	raw_values.clear();
	skip_white(f);
	if (f.eof() || ! isalpha(f.peek())) return false;
	// read id string
	do {
		id += f.get();
	} while ((! f.eof()) && isalpha(f.peek())); 
	do {
		skip_white(f);
		if (f.eof() || f.peek() != '[') break;
		f.get();
		string s = "";
		bool escape = false;
		int line_break = 0;
		while ((! f.eof()) && (f.peek() != ']' || escape)) {
			char c = f.get();
			if (c == '\r') {
				if (line_break == 1) {
					line_break = 0;
					continue;
				}
				c = '\n';
				line_break = 2;
			} else if (c == '\n') {
				if (line_break ==2) {
					line_break = 0;
					continue;
				}
				line_break = 1;
			} else line_break = 0;
			if (escape) {
				escape = false;
				if (c == '\n') continue;
			} else escape = c == '\\';
			if (! escape) s += c;
		}
		if (f.eof()) {
			std::cerr << "incomplete value field!" << std::endl;
		} else f.get();
		raw_values.push_back(s);
	} while (true);
	return true;
}

void Prop::save(ostream & f) const
{
	f << id;
	for (vector<string>::const_iterator i = raw_values.begin(); i != raw_values.end(); i ++) {
		f << '[';
		for (unsigned j = 0; j < i->length(); j ++) {
			if ((* i)[j] == ']' || (* i)[j] == '\\') f << '\\';
			f << (* i)[j];
		}
		f << ']';
	}
}

const string & Prop::get_id() const
{
	return id;
}

int Prop::as_int() const
{
	if (raw_values.size() != 1) return 0;
	istringstream s(raw_values[0]);
	int i;
	s >> i;
	return i;
}

double Prop::as_real() const
{
	if (raw_values.size() != 1) return 0;
	istringstream s(raw_values[0]);
	double d;
	s >> d;
	return d;
}

go::Loc Prop::as_loc(unsigned yf) const
{
	string s = raw_values[0];
	if (s.length() < 2) return go::Loc();
	int x = s[0] - 'a';
	int y = s[1] - 'a';
	// compressed points can not be handle here!
	if (yf == 0) return go::Loc(x, y);
	y = yf - y - 1;
	if (go::Loc(x, y).in_range(yf)) return go::Loc(x, y);
	return go::Loc();
}

vector<go::Loc> Prop::ar_loc(unsigned yf) const
{
	vector<go::Loc> r;

	for (vector<string>::const_iterator i = raw_values.begin();
	     i != raw_values.end(); i ++) {
		string s = * i;
		if (s.length() < 2) continue;
		int x = s[0] - 'a';
		int y = (yf == 0 ? s[1] - 'a' : yf + 'a' - s[1] - 1);
		if (s.length() == 2) {
			r.push_back(go::Loc(x, y));
			continue;
		}
		if (s[2] == ':') { //compressed point
			if (s.length() < 5) continue;
			int x2 = s[3] - 'a';
			int y2 = (yf == 0 ? s[4] - 'a' : yf + 'a' - s[4] - 1);
			if (yf != 0) {
				int t = y2;
				y2 = y;
				y = t;
			}
			for (int xx = x; xx <= x2; xx++) for (int yy = y; yy <= y2; yy ++) r.push_back(go::Loc(xx, yy));
			continue;
		}
	}
	return r;
}

char Prop::as_char() const
{
	return raw_values[0][0];
}

const string & Prop::as_string() const
{
	return raw_values[0];
}

go::Proper::Label Prop::as_label(unsigned yf) const
{
	go::Proper::Label l;
	std::string s = raw_values[0];
	l.loc = go::Loc(s[0] - 'a', yf == 0 ? s[1] - 'a' : yf + 'a' - s[1] - 1);
	// assert(s[2] == ':');
	l.label = s.substr(3);
	return l;
}

std::vector<go::Proper::Label> Prop::ar_label(unsigned yf) const
{
	std::vector<go::Proper::Label> la;
	for (unsigned i = 0; i < raw_values.size(); i ++) {
		std::string s = raw_values[i];
		go::Proper::Label l;
		l.loc = go::Loc(s[0] - 'a', yf == 0 ? s[1] - 'a' : yf + 'a' - s[1] - 1);
		// assert(s[2] == ':');
		l.label = s.substr(3);
		la.push_back(l);
	}
	return la;
}
